from aioplus.internal.aall import aall
from aioplus.internal.aany import aany
from aioplus.internal.abatched import abatched
from aioplus.internal.achain import achain
from aioplus.internal.acount import acount
from aioplus.internal.acycle import acycle
from aioplus.internal.aempty import aempty
from aioplus.internal.aenumerate import aenumerate
from aioplus.internal.afirst import afirst
from aioplus.internal.ahead import ahead
from aioplus.internal.aislice import aislice
from aioplus.internal.alast import alast
from aioplus.internal.alen import alen
from aioplus.internal.amax import amax
from aioplus.internal.amin import amin
from aioplus.internal.aminmax import aminmax
from aioplus.internal.anextify import anextify
from aioplus.internal.anth import anth
from aioplus.internal.apairwise import apairwise
from aioplus.internal.apostpend import apostpend
from aioplus.internal.aprepend import aprepend
from aioplus.internal.arange import arange
from aioplus.internal.arepeat import arepeat
from aioplus.internal.areversed import areversed
from aioplus.internal.asum import asum
from aioplus.internal.atabulate import atabulate
from aioplus.internal.atail import atail
from aioplus.internal.atriplewise import atriplewise
from aioplus.internal.awaitify import awaitify
from aioplus.internal.awindowed import awindowed
from aioplus.internal.azip import azip
from aioplus.internal.caller_thread_executor import CallerThreadExecutor


__author__ = "Sergei Y. Bogdanov <syubogdanov@outlook.com>"
__version__ = "0.7.0"

__all__: list[str] = [
    "CallerThreadExecutor",
    "aall",
    "aany",
    "abatched",
    "achain",
    "acount",
    "acycle",
    "aempty",
    "aenumerate",
    "afirst",
    "ahead",
    "aislice",
    "alast",
    "alen",
    "amax",
    "amin",
    "aminmax",
    "anextify",
    "anth",
    "apairwise",
    "apostpend",
    "aprepend",
    "arange",
    "arepeat",
    "areversed",
    "asum",
    "atabulate",
    "atail",
    "atriplewise",
    "awaitify",
    "awindowed",
    "azip",
]


CallerThreadExecutor.__module__ = "aioplus"
aall.__module__ = "aioplus"
aany.__module__ = "aioplus"
abatched.__module__ = "aioplus"
achain.__module__ = "aioplus"
acount.__module__ = "aioplus"
acycle.__module__ = "aioplus"
aempty.__module__ = "aioplus"
aenumerate.__module__ = "aioplus"
afirst.__module__ = "aioplus"
ahead.__module__ = "aioplus"
aislice.__module__ = "aioplus"
alast.__module__ = "aioplus"
alen.__module__ = "aioplus"
amax.__module__ = "aioplus"
amin.__module__ = "aioplus"
aminmax.__module__ = "aioplus"
anextify.__module__ = "aioplus"
anth.__module__ = "aioplus"
apairwise.__module__ = "aioplus"
apostpend.__module__ = "aioplus"
aprepend.__module__ = "aioplus"
arange.__module__ = "aioplus"
arepeat.__module__ = "aioplus"
areversed.__module__ = "aioplus"
asum.__module__ = "aioplus"
atabulate.__module__ = "aioplus"
atail.__module__ = "aioplus"
atriplewise.__module__ = "aioplus"
awaitify.__module__ = "aioplus"
awindowed.__module__ = "aioplus"
azip.__module__ = "aioplus"
